#include "icselinc.h"

CIconSelDialog::CIconSelDialog(PICONSELINFO pInfo, HINSTANCE hInst)
{
   m_hInst = hInst;
   m_pUserInfo = pInfo;
   m_szIconFile[0] = 0;
   m_hImgLarge = m_hImgSmall = NULL;
}

// Called to handle the WM_INITDIALOG message
BOOL CIconSelDialog::OnCreate(HWND hDlg)
{
   m_hWnd = hDlg;
   m_hWndList = GetDlgItem(m_hWnd, IDC_ICONLIST);

   m_iLargeIconWidth  = GetSystemMetrics(SM_CXICON);
   m_iLargeIconHeight = GetSystemMetrics(SM_CYICON);
   m_iSmallIconWidth  = GetSystemMetrics(SM_CXSMICON);
   m_iSmallIconHeight = GetSystemMetrics(SM_CYSMICON);

   // Set the caption if supplied
   if (m_pUserInfo->pszCaption)
      SetWindowText(m_hWnd, m_pUserInfo->pszCaption);

   // Hide the browse button if not needed
   if ((m_pUserInfo->dwFlags & ICSEL_HIDEBROWSEBTN))
      ShowWindow(GetDlgItem(m_hWnd, IDC_BROWSE), SW_HIDE);

   // Hide the large/small radio buttons if not needed
   if (!(m_pUserInfo->dwFlags & ICSEL_LARGEICONS) ||
       !(m_pUserInfo->dwFlags & ICSEL_SMALLICONS)) {
      ShowWindow(GetDlgItem(m_hWnd, IDC_LARGE), SW_HIDE);
      ShowWindow(GetDlgItem(m_hWnd, IDC_SMALL), SW_HIDE);
   }
   else
      Button_SetCheck((m_pUserInfo->bLarge) ?
           GetDlgItem(m_hWnd,IDC_LARGE) :
           GetDlgItem(m_hWnd, IDC_SMALL), TRUE);

   // Set the correct initial icon size
   if (!(m_pUserInfo->dwFlags & ICSEL_LARGEICONS))
      m_pUserInfo->bLarge = FALSE;
   if (!(m_pUserInfo->dwFlags & ICSEL_SMALLICONS))
      m_pUserInfo->bLarge = TRUE;

   // Hide the filename and edit box if not required.The selection
   // listbox will be enlarged to cover the space left when the 2
   // controls are removed
   if (m_pUserInfo->dwFlags & ICSEL_HIDEFILENAME) {
      RECT rctText, rctList;
      GetWindowRect(GetDlgItem(m_hWnd, IDC_LABEL_FNAME), &rctText);
      GetWindowRect(GetDlgItem(m_hWnd, IDC_ICONLIST), &rctList);

      ScreenToClient(m_hWnd, (LPPOINT)&rctText.left);
      ScreenToClient(m_hWnd, (LPPOINT)&rctText.right);
      ScreenToClient(m_hWnd, (LPPOINT)&rctList.left);
      ScreenToClient(m_hWnd, (LPPOINT)&rctList.right);

      ShowWindow(GetDlgItem(m_hWnd, IDC_LABEL_FNAME), SW_HIDE);
      ShowWindow(GetDlgItem(m_hWnd, IDC_FILENAME), SW_HIDE);
      SetWindowPos(GetDlgItem(m_hWnd, IDC_ICONLIST), NULL,
            rctList.left, rctText.top, rctList.right - rctList.left,
            rctList.bottom - rctText.top, SWP_NOZORDER);
   }

   // Figure out the name of the file to extract from
   if (m_pUserInfo->dwFlags & ICSEL_USEHINSTANCE)
      GetModuleFileName(m_pUserInfo->hInstance, m_szIconFile,
                        sizeof(m_szIconFile));
   else if (m_pUserInfo->pszFile)
      strcpy(m_szIconFile, m_pUserInfo->pszFile);

   BuildImageLists();
   PopulateList();

   return TRUE;
}

// Builds the 2 image lists containing the icons
void CIconSelDialog::BuildImageLists()
{
   // Get the number of icons in the file
   int iNumIcons = ExtractIconEx(m_szIconFile, -1, NULL, NULL, 0);
   if (!iNumIcons) return;

   // Destroy any previous image lists
   if (m_hImgLarge) {
      ImageList_Destroy(m_hImgLarge);
      m_hImgLarge = NULL;
   }

   if (m_hImgSmall) {
      ImageList_Destroy(m_hImgSmall);
      m_hImgSmall = NULL;
   }

   // Create 2 Image lists (1 for large and one for small)
   m_hImgLarge = ImageList_Create(m_iLargeIconWidth,
         m_iLargeIconHeight, ILC_COLORDDB | ILC_MASK, 0, iNumIcons);
   if (!m_hImgLarge) return;

   m_hImgSmall = ImageList_Create(m_iSmallIconWidth,
         m_iSmallIconHeight,ILC_COLORDDB | ILC_MASK,0, iNumIcons);
   if (!m_hImgSmall) {
      ImageList_Destroy(m_hImgLarge);
      return;
   }

   // Extract the Icons from the file in one go
   HICON FAR *pLargeIcons = NULL;
   HICON FAR *pSmallIcons = NULL;

   if (m_pUserInfo->dwFlags & ICSEL_LARGEICONS) {
      pLargeIcons = new HICON[iNumIcons];
      if (!pLargeIcons)
         return;
   }

   if (m_pUserInfo->dwFlags & ICSEL_SMALLICONS) {
      pSmallIcons = new HICON[iNumIcons];
      if (!pSmallIcons) {
         delete pLargeIcons;
         return;
      }
   }

   ExtractIconEx(m_szIconFile, 0, pLargeIcons, pSmallIcons,
                 iNumIcons);

   // Add the icons to the image lists
   for (int i = 0;i < iNumIcons;i++) {
      if (pLargeIcons) {
         if (*(pLargeIcons + i)) {
            ImageList_AddIcon(m_hImgLarge, *(pLargeIcons + i));
            DestroyIcon(*(pLargeIcons + i));
         }
      }

      if (pSmallIcons) {
         if (*(pSmallIcons + i)) {
            ImageList_AddIcon(m_hImgSmall, *(pSmallIcons + i));
            DestroyIcon(*(pSmallIcons + i));
         }
      }
   }

   if (pLargeIcons)
      delete pLargeIcons;
   if (pSmallIcons)
      delete pSmallIcons;

   m_iNumIcons = iNumIcons;

   if (!(m_pUserInfo->dwFlags & ICSEL_HIDEFILENAME))
      SetDlgItemText(m_hWnd, IDC_FILENAME, m_szIconFile);
}

// Fills the listbox with a dummy entry for each icon present
void CIconSelDialog::PopulateList()
{
   HWND  hWndList = GetDlgItem(m_hWnd, IDC_ICONLIST);
   ListBox_ResetContent(hWndList);

   ListBox_SetColumnWidth(hWndList,
               (m_pUserInfo->bLarge) ?
                  (m_iLargeIconWidth + (LARGE_SPACING * 2)) :
                  (m_iSmallIconWidth + (SMALL_SPACING * 2)));

   ListBox_SetItemHeight(hWndList, 0,
               (m_pUserInfo->bLarge) ?
                  (m_iLargeIconHeight + (LARGE_SPACING * 2)) :
                  (m_iSmallIconHeight + (SMALL_SPACING * 2)));

   for (int i = 0;i < m_iNumIcons;i++)
      ListBox_AddString(hWndList, NULL);
   ListBox_SetCurSel(hWndList, 0);
}

// Called to draw each icon in the list box
void CIconSelDialog::OnDrawItem(LPDRAWITEMSTRUCT lpdis)
{
   if (!ListBox_GetCount(m_hWndList))
      return;

   int   iRowHeight = lpdis->rcItem.bottom - lpdis->rcItem.top;

   // Draw the selection box depending on whether the item
   // is selected or not.
   if (lpdis->itemState & ODS_SELECTED) {
      SetBkColor(lpdis->hDC, GetSysColor(COLOR_HIGHLIGHT));
      SetTextColor(lpdis->hDC, GetSysColor(COLOR_HIGHLIGHTTEXT));
   }
   else {
      SetBkColor(lpdis->hDC, GetSysColor(COLOR_WINDOW));
      SetTextColor(lpdis->hDC, GetSysColor(COLOR_WINDOWTEXT));
   }

   ExtTextOut(lpdis->hDC, lpdis->rcItem.left, lpdis->rcItem.top,
      ETO_CLIPPED | ETO_OPAQUE, &lpdis->rcItem, "", 0, NULL );

   // Draw the Icon
   if (m_pUserInfo->bLarge)
      ImageList_Draw(m_hImgLarge, lpdis->itemID, lpdis->hDC,
            lpdis->rcItem.left + LARGE_SPACING,
            lpdis->rcItem.top +  LARGE_SPACING, ILD_TRANSPARENT );
   else
      ImageList_Draw(m_hImgSmall, lpdis->itemID, lpdis->hDC,
            lpdis->rcItem.left + SMALL_SPACING,
            lpdis->rcItem.top +  SMALL_SPACING, ILD_TRANSPARENT );
}

// Called when the 'Large' and 'Small' radio buttons are pressed
void CIconSelDialog::OnSizeChange(BOOL bLarge)
{
   if (m_pUserInfo->bLarge == bLarge)
      return;
   m_pUserInfo->bLarge = bLarge;

   ListBox_SetColumnWidth(m_hWndList,
               (m_pUserInfo->bLarge) ?
                  (m_iLargeIconWidth + (LARGE_SPACING * 2)) :
                  (m_iSmallIconWidth + (SMALL_SPACING * 2)));

   ListBox_SetItemHeight(m_hWndList, 0,
               (m_pUserInfo->bLarge) ?
                  (m_iLargeIconHeight + (LARGE_SPACING * 2)) :
                  (m_iSmallIconHeight + (SMALL_SPACING * 2)));

   ListBox_SetTopIndex(m_hWndList, ListBox_GetCurSel(m_hWndList));
}

// Called when the OK button is pressed or the ENTER key is pressed
void CIconSelDialog::OnOK()
{
   // See was the user typing a file name in the edit field
   HWND hWndFocus = GetFocus();
   if (hWndFocus == GetDlgItem(m_hWnd, IDC_FILENAME)) {
      char  szFile[MAX_PATH];
      GetDlgItemText(m_hWnd, IDC_FILENAME, szFile, sizeof(szFile));
      SelectNewIconFile(szFile);
      return;
   }

   // Store the selected Icons in the users structure
   m_pUserInfo->iSelIcon = ListBox_GetCurSel(m_hWndList);
   if (LB_ERR == m_pUserInfo->iSelIcon)
      return;

   if (m_pUserInfo->dwFlags & ICSEL_LARGEICONS)
      m_pUserInfo->hIconLarge = ImageList_GetIcon(m_hImgLarge,
         m_pUserInfo->iSelIcon, ILD_TRANSPARENT);

   if (m_pUserInfo->dwFlags & ICSEL_SMALLICONS)
      m_pUserInfo->hIconSmall = ImageList_GetIcon(m_hImgSmall,
         m_pUserInfo->iSelIcon, ILD_TRANSPARENT);

   // Copy the current file name into the users buffer
   strncpy(m_pUserInfo->pszFile, m_szIconFile,
           m_pUserInfo->nMaxFileSize);
   *(m_pUserInfo->pszFile + m_pUserInfo->nMaxFileSize) = 0;

   EndDialog(m_hWnd, IDOK);
}

// Called when the Browse button is pressed
void CIconSelDialog::OnBrowse()
{
   OPENFILENAME ofn;
   char szFile[MAX_PATH] = "", szFilters[MAX_PATH];

   memset(&ofn, 0, sizeof(ofn));
   ofn.lStructSize = sizeof(ofn);
   ofn.hwndOwner = m_hWnd;
   ofn.lpstrFile = szFile;
   ofn.nMaxFile = sizeof(szFile);
   ofn.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;

   // Load the list of filter strings
   if (!LoadString(m_hInst, IDS_FILTERS, szFilters,
                   sizeof(szFilters)))
      return;

   LPSTR pszSep = strchr(szFilters, '|');
   while(pszSep) {
      *pszSep++ = 0;
      pszSep = strchr(pszSep, '|');
   }

   ofn.lpstrFilter = szFilters;

   // Display the file open dialog
   if (IDOK != GetOpenFileName(&ofn))
      return;

   SelectNewIconFile(szFile);
}

// Changes the currennt file. Causes the icons in the new file to
// be displayed
BOOL CIconSelDialog::SelectNewIconFile(LPSTR pszFile)
{
   // Get the number of icons in the file
   int iNumIcons = (int)ExtractIcon(m_hInst, pszFile, -1);

   char  szError[128], szTitle[64];
   if (!iNumIcons) {
      LoadString(m_hInst, IDS_NOICONS, szError, sizeof(szError));
      LoadString(m_hInst, IDS_ERROR, szTitle, sizeof(szTitle));
      MessageBox(m_hWnd, szError, szTitle, MB_OK | MB_ICONSTOP);
      return FALSE;
   }

   strcpy(m_szIconFile, pszFile);
   BuildImageLists();
   PopulateList();
   return TRUE;
}