#define UNICODE

#define _WIN32_WINNT 0x0400

#include <windows.h>
#include <winioctl.h>
#include <stdio.h>
#include <ctype.h>
#include <wchar.h>
#include <winsock2.h>
#include <ws2tcpip.h>
#include "nicip.h"

BOOL  GetInterfaceEntryByIndex(HANDLE deviceHandle, ULONG ulIndex,
                                  IFEntry *ifEntry, ULONG *ulSize)
{
    BOOL    status;
    ULONG   i;
    DWORD   dwReturn;
    TCP_REQUEST_QUERY_INFORMATION_EX    tcpRequest;

    status = TRUE;
    for(i = 0;status == TRUE && ifEntry->if_index != ulIndex;i++) {

        ZeroMemory(ifEntry, *ulSize );

        INIT_TCP_REQUEST(tcpRequest, i, IF_ENTITY, 
                            INFO_CLASS_PROTOCOL,
                            INFO_TYPE_PROVIDER, IF_MIB_STATS_ID)

        status = DeviceIoControl(deviceHandle,
                            IOCTL_TCP_QUERY_INFORMATION_EX,
                            &tcpRequest,  sizeof(tcpRequest),
                            ifEntry,      *ulSize,
                            &dwReturn,    NULL);

    }

    *ulSize = dwReturn;
    return  status;

}/* End of GetInterfaceEntryByIndex() */


ULONG GetTotalNoOfIps(HANDLE deviceHandle, TDIEntityID entityId)
/*
 *  For the given TDIEntityID,
 *  this will determine the number of IP addresses available.
 */
{
    
    DWORD       dwReturn;
    IPSNMPInfo  snmpInfo;
    TCP_REQUEST_QUERY_INFORMATION_EX    tcpRequest;
    
    ZeroMemory(&snmpInfo,   sizeof(snmpInfo));

    INIT_TCP_REQUEST(tcpRequest, entityId.tei_instance, 
                        entityId.tei_entity,INFO_CLASS_PROTOCOL,
                        INFO_TYPE_PROVIDER, IP_MIB_STATS_ID)

    DeviceIoControl(deviceHandle, IOCTL_TCP_QUERY_INFORMATION_EX,
                        &tcpRequest, sizeof(tcpRequest),
                        &snmpInfo,   sizeof(snmpInfo),
                        &dwReturn,   NULL);

    return    snmpInfo.ipsi_numaddr;

}/* End of GetTotalNoOfIps() */

void PrintIpAndNicList(void)
/*
 *    This routine queries the TCP Transport Driver for Network
 *    Interface Card and its corresponding IP Address and displays
 *    the results.
 */
{
    /* Variables for CreateFile(), DeviceIoControl()  */
    BOOL    status;
    HANDLE  deviceHandle;  /* TCP device Handle */
    TCHAR   dosName[256];  /* Holds DOS Device Name*/
    DWORD   dwReturn;    
    ULONG   uNoOfIpAddress;
    ULONG   i,j;
    ULONG   entityCount;
    ULONG   entityBufferLength;
    ULONG   entityType;
    ULONG   addressBufferLength;
    CHAR    ifentryBuffer[sizeof(IFEntry) + MAX_IFDESCR_LEN];

    /* TCP request related structures declared in IpNic.h   */
    
    IFEntry       * ifentry;
    IPAddrEntry   addressBuffer[60];
    IPAddrEntry   * addressScan;
    TDIEntityID   entityBuffer[25];
    TDIEntityID   * entityScan;
    
    TCP_REQUEST_QUERY_INFORMATION_EX    tcpRequest;

    /* Init Variables.  */

    deviceHandle            = NULL;
    entityBufferLength      = sizeof entityBuffer;
    addressBufferLength     = sizeof addressBuffer;

    /*
     * Create a DOS Name so that it can be opened using CreateFile()
     */
    status = DefineDosDevice(DDD_RAW_TARGET_PATH,
                                DOS_TCP_NAME ,WIN32_TCP_NAME);
    if(status == FALSE) {

        wprintf(L"Can't create Symbolic Link to %s.\n", 
                                        WIN32_TCP_NAME);
        return;
    }
    /* Open a handle to the TCP/IP device.   */
    wsprintf(dosName, L"%s%s",DOS_DEVICE_PREFIX,DOS_TCP_NAME);
    deviceHandle = CreateFile (dosName,
                             GENERIC_READ | GENERIC_WRITE, 
                             FILE_SHARE_READ | FILE_SHARE_WRITE,
                             NULL, OPEN_EXISTING, 
                             FILE_ATTRIBUTE_NORMAL,NULL);
    if( deviceHandle == INVALID_HANDLE_VALUE ) {
        wprintf(L"Can't create handle to %s\n", dosName);
        return;
    }

    ZeroMemory(&tcpRequest,sizeof tcpRequest);

    /* Get all the entity IDs  available  */
    INIT_TCP_REQUEST(tcpRequest, 0, GENERIC_ENTITY, 
                        INFO_CLASS_GENERIC,
                        INFO_TYPE_PROVIDER, ENTITY_LIST_ID)
    status = DeviceIoControl(deviceHandle, 
                            IOCTL_TCP_QUERY_INFORMATION_EX,
                             &tcpRequest,  sizeof(tcpRequest),
                             entityBuffer, entityBufferLength,
                             &dwReturn,    NULL);    
    entityCount = entityBufferLength / sizeof(entityBuffer[0]);

    /* Scan the entities looking for Network Layer Entity  */
    for( i = 0, entityScan = entityBuffer ;    
                    i < entityCount ; i++, entityScan++ ) {
        if( entityScan->tei_entity != CL_NL_ENTITY )
           continue;
        /* Network Layer Entity found.  Now go for IP Entity  */
        INIT_TCP_REQUEST(tcpRequest, entityScan -> tei_instance, 
                        entityScan->tei_entity, INFO_CLASS_GENERIC,
                        INFO_TYPE_PROVIDER, ENTITY_TYPE_ID)
        status = DeviceIoControl(deviceHandle, 
                                IOCTL_TCP_QUERY_INFORMATION_EX,
                                &tcpRequest,  sizeof(tcpRequest),
                                &entityType,  sizeof(entityType),
                                &dwReturn,    NULL);    
        if( status == FALSE ) {
            printf("Error : ENTITY_TYPE_ID IOCTL failed\n");
            break;
        }
        if( entityType != CL_NL_IP )
            continue;
        uNoOfIpAddress = GetTotalNoOfIps(deviceHandle, *entityScan);
        if( uNoOfIpAddress <= 0 )
            continue;
        /*
         * This interface has addresses. Calculate the buffer size
         * needed to store them.
         */
        addressBufferLength = uNoOfIpAddress * 
                                    sizeof(addressBuffer[0]);
        if( addressBufferLength > sizeof(addressBuffer) ) {
            printf("Error : Not enough memory for AddressBuffer\n");
            break;
        } 

        /* Got No. of interfaces.  Get Entity Addresses      */
        INIT_TCP_REQUEST(tcpRequest, entityScan->tei_instance, 
                        entityScan->tei_entity, INFO_CLASS_PROTOCOL,
                        INFO_TYPE_PROVIDER, 
                        IP_MIB_ADDRTABLE_ENTRY_ID)
        status = DeviceIoControl(deviceHandle,  
                                IOCTL_TCP_QUERY_INFORMATION_EX,
                                &tcpRequest,   sizeof(tcpRequest),
                                addressBuffer, addressBufferLength,
                                &dwReturn,     NULL);    
        if( status == FALSE ) {
            printf(" Error : IP_MIB_ADDRTABLE_ENTRY failed\n");
            break;
        }

        ifentry        = (PVOID)ifentryBuffer;

        puts("---------------------------------------------------");
        puts("  IP Address     Network Address       Description ");
        puts("---------------------------------------------------");

        /* Now scan the list  */
        for( j = 0, addressScan = addressBuffer ;
                     j < uNoOfIpAddress ;j++, addressScan++ ) {
                /* Skip entries that don't have IP addresses yet. */
                if( addressScan->iae_addr == 0    ||
                addressScan->iae_addr == htonl(INADDR_LOOPBACK))
                    continue;
                dwReturn = sizeof(ifentryBuffer);
                status = GetInterfaceEntryByIndex(deviceHandle, 
                                    addressScan->iae_index,
                                    ifentry, &dwReturn);
                if(status == FALSE)
                    continue;

                /*
                 * Display <IP, NIC, Description> 
                 */

                DISPLAY_IP_ADDRESS(((PUCHAR)&addressScan->iae_addr));
                printf("\t");

                DISPLAY_NIC_ADDRESS(ifentry->if_physaddr);
        
                printf("  %*s\n",ifentry->if_descrlen,
                                    ifentry->if_descr);

                

            }/* End of IP Scan */

    }/* End of Entity Scan*/

    if( deviceHandle != NULL ) {
        CloseHandle ( deviceHandle );
    }

    /* Remove the DOS Name  */
    DefineDosDevice(DDD_REMOVE_DEFINITION, DOS_TCP_NAME,NULL);

    return ;
}   /* End of PrintIpAndNicList() */

int main(void)
{
    PrintIpAndNicList();
    getchar();
    return 0;
}
